#!/usr/bin/ruby
# coding: utf-8
# Written by Teruaki Kitasuka, Hiroshima University, 2018.

@gm = 32; @gn = 8; @gr = 3
@vs1 = []; @vs2 = [[5, 7], [6, 3], [12, 2], [12, 3], [16, 1]]

@n = @gm * @gn
@d = @vs1.length * 1 + @vs2.length * 2
# @vs1 = []; @vs2 = [[5, 7], [6, 3], [12, 2], [12, 3], [16, 1]] # n256d10-k3l2.5686274509803924-gm32gn8gr3; [1, 10, 90, 155]; @vs2len = [8, 8, 8, 8, 16] k3l 2.5686274509803924 - 2.6666666666666665 (end of 500 x 20 loops)

def semiDirectProduct(x1, y1, x2, y2)
  [(x1 + (@gr ** y1) * x2) % @gm, (y1 + y2) % @gn]
end

def cayleyGraph(vs1, vs2)
  @e = (0...@n).map { |i| [] }
  (0...@gm).each { |mi|
    (0...@gn).each { |ni|
      (vs1 + vs2).each { |mv, nv|
        mj, nj = semiDirectProduct(mi, ni, mv, nv)
        i = mi + @gm * ni
        j = mj + @gm * nj
        if @e[i].index(j) == nil # 片方向の枝
          addEdge(@e, i, j)
        end
      }
    }
  }
  @e.map! { |ei| ei.sort }
end

def addEdge(e, i, j)
  e[i] << j
  e[j] << i
end

def save(e, n, d, vs1, vs2)
  return if e.length != n
  vs1s = vs1.map { |v| "(#{v.join(',')})" }.join('')
  vs1s += ' | ' if vs1s.length > 0
  vs2s = vs2.map { |v| "(#{v.join(',')})" }.join('')
  puts "# Cayley graph"
  puts "# Group m = #{@gm}, n = #{@gn}, r = #{@gr}; Quotient B(#{vs1.length}, #{vs2.length})"
  puts "# Voltages [#{vs1s}#{vs2s}]"
  (0...n).each { |i|
    e[i].sort.each { |j|
      next if i > j
      puts "#{i} #{j}"
    }
  }
end

cayleyGraph(@vs1, @vs2)
save(@e, @n, @d, @vs1, @vs2)
